# coding: utf-8

import pymongo
import logging

DB_NAME = 'junta'
db = pymongo.Connection()[DB_NAME]

def get_names():
    """ Получение списка имен всех агентов
    """
    machines = db.machines.find()
    return [a['name'] for a in machines]

def get_ids():
    """ Получение списка имен всех агентов
    """
    machines = db.machines.find()
    return [a['_id'] for a in machines]

def login(machine_id):
    """ Функция выполняется когда подключается агент.
        Если он новый, он записывается в бд и ему присваивается имя равное его id
    """
    machine = db.machines.find_one({'_id': machine_id})
    if not machine:
        name = machine_id
    else:
        name = machine['name']    
    db.machines.save({'_id': machine_id,        # уникальный идентификатор сервера 
                    'name': name,               # имя сервера
                    'recent_installation': '',  # id последней инсталляции
                    'status': 'Online',         # статус
                    'level': 1})                # уровень доступа, который должен быть
                                                # у пользователя для управления этим сервером
                                
def get_name(machine_id):
    """ Получение имени агента по его id
    """
    machine = db.machines.find_one({'_id': machine_id})
    if machine:
        return machine['name']
    else:
        return None
    
def rename(machine_id, new_name):
    """ Переименнование агента
    """
    machine = db.machines.find_one({'_id': machine_id})
    machine['name'] = new_name
    db.machines.save(machine) 
    
def get_recent_installation(machine_id):
    """ Возвращает id последней инсталляции на сервер
    """
    return db.machines.find_one({'_id': machine_id})['recent_installation']
    
def set_recent_installation(machine_id, installation_id=0):
    """ Установка id последней инсталляции на сервер
    """
    machine = db.machines.find_one({'_id': machine_id})
    machine['recent_installation'] = installation_id
    db.machines.save(machine)
    
def get_status(machine_id):
    """ Установка статуса для агента
    """
    machine = db.machines.find_one({'_id': machine_id})
    if machine is not None:
        return machine['status']
    else:
        return 'Offline'
    
def set_status(machine_id, status):
    """ Установка статуса для агента
    """
    machine = db.machines.find_one({'_id': machine_id})
    machine['status'] = status
    db.machines.save(machine)
    
def get_level(machine_id):
    """ Получение уровеня доступа, который должен быть
        у пользователя для управления этим сервером
    """
    machine = db.machines.find_one({'_id': machine_id})
    if machine is not None:
        return machine['level']
    else:
        return 1 # уровень не установлен, дефолтный (у юзеров дефолтный уровень = 0)

def set_level(machine_id, level):
    """ Установка уровеня доступа, который должен быть
        у пользователя для управления этим сервером
    """
    machine = db.machines.find_one({'_id': machine_id})
    machine['level'] = level
    db.machines.save(machine)

def delete(machine_id):
    """ Удаление агента из бд
    """
    db.machines.remove({'_id': machine_id})


def save_log(server_id, msg_type, msg, n):
    """ Сохраняет сообщение в лог.
    """
    installation_id = get_recent_installation(server_id)
    if installation_id == 0:
        logging.error("error in save_log")
        return
    db.log.save({'installation_id': installation_id,
                 'type': msg_type,
                 'message': msg,
                 'n': n})
    
def set_installation_status(server_id, status):
    """ Устанавливает новый статус инсталляции.
        Для какой инсталляции, смотрит recent_installation у агента
    """
    installation_id = get_recent_installation(server_id)
    if installation_id == 0:
        logging.error("error in set_installation_status")
        return
    to_change = db.installed.find_one({'_id': installation_id})
    to_change['status'] = status
    db.installed.save(to_change)


"""    Сервера и группы    """
def get_all_groups():
    groups = set() 
    for g in db.groups.find():
        groups.add(g['name'])
    return list(groups)

def get_groups_not_in(machine_id):
    groups = set() 
    for g in db.groups.find():
        groups.add(g['name'])
    groups_in = set(get_groups(machine_id))
    return groups - groups_in

def get_groups(machine_id):
    return [g['name'] for g in db.groups.find({'machine_id': machine_id})]

def get_machines_in_group(group_name):
    return [g['machine_id'] for g in db.groups.find({'name': group_name})]

def set_group(machine_id, group_name):
    db.groups.save({'_id': machine_id + "_" + group_name,
                    'machine_id': machine_id,
                    'name': group_name })

def remove_from_group(machine_id, group_name):
    db.groups.remove({'machine_id': machine_id, 'name': group_name})


"""    Сервера и права пользователей    """
def get_perms(machine_id):
    return [(p['user_name'], p['perm']) for p in db.perms.find({'machine_id': machine_id})]
    
def set_perm_for_user(machine_id, user_name, perm):
    db.perms.save({'_id': machine_id + "_" + user_name,
                   'machine_id': machine_id, 
                   'user_name': user_name,
                   'perm': perm })

def set_perm_for_user_on_group(group_name, user_name, perm):
    for machine_id in get_machines_in_group(group_name):
        set_perm_for_user(machine_id, user_name, perm)

def set_perm_for_user_on_all(user_name, perm):
    for machine_id in get_ids():
        set_perm_for_user(machine_id, user_name, perm)


"""    Сервера и атрибуты    """
def get_attrs(machine_id):
    return [(a['name'], a['default']) for a in db.attrs.find({'machine_id': machine_id})]

def set_attr(machine_id, attr_name, default=''):
    db.attrs.save({'_id': machine_id + "_" + attr_name,
                   'machine_id': machine_id, 
                   'name': attr_name,
                   'default': default })

def delete_attr(machine_id, attr_name):
    db.attrs.remove({'machine_id': machine_id, 
                     'name': attr_name})

def set_attr_on_group(group_name, attr_name, default=''):
    for machine_id in get_machines_in_group(group_name):
        set_attr(machine_id, attr_name, default)

def set_attr_on_all(attr_name, default=''):
    for machine_id in get_ids():
        set_attr(machine_id, attr_name, default)